'use strict'

// Imports
const Me = imports.misc.extensionUtils.getCurrentExtension()
const Effect = Me.imports.effect;
const { Meta } = imports.gi;

// Which window to blur
const WindowTitlePrefix = 'ext'
const WindowClass = 'ext'

class Extension {

  constructor() {
    this.windows = []
  }

  enable() {

    // Register events
    this.idCreated = global.display.connect('window-created', (_, w) => this.onWindowCreated(w))

    // Check existing windows
    const workspaces = global.workspace_manager.get_n_workspaces()
    for (let i = 0; i < workspaces; i++) {
      const workspace = global.workspace_manager.get_workspace_by_index(i)
      const windows = workspace.list_windows()
      for (let j = 0; j < windows.length; j++) {
        this.onWindowCreated(windows[j])
      }
    }

  }

  disable() {

    // Unregister events
    global.display.disconnect(this.idCreated)

    // Disable blur on all windows
    this.windows.forEach(meta_window => {
      meta_window.destroy()
    })

    // Clear windows
    this.windows = []

  }

  onWindowCreated(meta_window) {

    // Check if window matches
    if (!this.isWindowMatch(meta_window)) {
      return
    }

    // Enable blur
    if (Meta.is_wayland_compositor()) {
      setTimeout(() => this.windowEnableBlur(meta_window), 50)
    } else {
      this.windowEnableBlur(meta_window)
    }

  }

  windowEnableBlur(meta_window) {

    // Enable blur and add to windows array
    const blurred_window = new Effect.BlurredWindow(meta_window, this)
    this.windows.push(blurred_window);

  }

  onWindowDestroyed(blur_window) {

    // Remove blur
    blur_window.destroy()

    // Remove from windows array
    let index = this.windows.indexOf(blur_window.meta_window);
    this.windows.splice(index, 1);

  }

  isWindowMatch(meta_window) {

    // Check if window class matches
    const wm_class = meta_window.get_wm_class()?.toUpperCase()
    if (wm_class == WindowClass.toUpperCase()) {
      return true
    }

    // Check if title matches
    const title = meta_window.get_title()?.toUpperCase()
    const title_prefix = WindowTitlePrefix.toUpperCase()
    if ((title == title_prefix) || (title && title.startsWith(title_prefix + ' '))) {
      return true
    }

    // No match
    return false

  }

}

function init() {
  return new Extension()
}
